'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _resolveImport = require('./resolve-import');

var _resolveImport2 = _interopRequireDefault(_resolveImport);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var isSpecialTypes = function isSpecialTypes(types, node) {
  return ['isMemberExpression', 'isProperty'].filter(function (type) {
    return types[type](node);
  }).length > 0;
};

exports.default = function (_ref) {
  var types = _ref.types;

  var axioms = void 0;
  var specified = void 0;
  var selectedAxioms = void 0;

  var importAxiom = function importAxiom(name, file) {
    if (!selectedAxioms[name]) {
      selectedAxioms[name] = file.addImport((0, _resolveImport2.default)(name), 'default');
    }

    return types.clone(selectedAxioms[name]);
  };

  var hasBindingOfType = function hasBindingOfType(scope, name, type) {
    return scope.hasBinding(name) && scope.getBinding(name).path.type === type;
  };

  var matchesAxiom = function matchesAxiom(path, name) {
    return axioms[name] && (hasBindingOfType(path.scope, name, 'ImportDefaultSpecified') || hasBindingOfType(path.scope, name, 'ImportNamespaceSpecifier'));
  };

  var matchesAxiomExport = function matchesAxiomExport(path, name) {
    return specified[name] && hasBindingOfType(path.scope, name, 'ImportSpecifier');
  };

  return {
    visitor: {
      Program: {
        enter: function enter() {
          axioms = {};
          specified = {};
          selectedAxioms = {};
        }
      },

      ImportDeclaration: function ImportDeclaration(path) {
        if (path.node.source.value === 'bw-axiom') {
          path.node.specifiers.forEach(function (spec) {
            if (types.isImportSpecifier(spec)) {
              specified[spec.local.name] = spec.imported.name;
            } else {
              axioms[spec.local.name] = true;
            }
          });

          path.remove();
        }
      },
      CallExpression: function CallExpression(path) {
        if (!types.isIdentifier(path.node.callee)) return;

        if (matchesAxiomExport(path, path.node.callee.name)) {
          path.node.callee = importAxiom(specified[path.node.callee.name], path.hub.file);
        }

        if (path.node.arguments) {
          path.node.arguments = path.node.arguments.map(function (argument) {
            return matchesAxiomExport(path, path.node.callee.name) ? importAxiom(specified[path.node.callee.name]) : argument;
          });
        }
      },
      MemberExpression: function MemberExpression(path) {
        if (!matchesAxiom(path, path.node.object.name)) return;

        var _importAxiom = importAxiom(path.node.property.name, path.hub.file),
            type = _importAxiom.type,
            name = _importAxiom.name;

        path.replaceWith({ type: type, name: name });
      },
      Property: function Property(path) {
        if (types.isIdentifier(path.node.key) && path.node.computed && matchesAxiomExport(path, path.node.key.name)) {
          path.node.key = importAxiom(specified[path.node.key.name], path.hub.file);
        }

        if (types.isIdentifier(path.node.value) && matchesAxiomExport(path, path.node.value.name)) {
          path.node.value = importAxiom(specified[path.node.value.name], path.hub.file);
        }
      },
      Identifier: function Identifier(path) {
        if (matchesAxiomExport(path, path.node.name) && !isSpecialTypes(types, path.parent)) {
          var _importAxiom2 = importAxiom(specified[path.node.name], path.hub.file),
              type = _importAxiom2.type,
              name = _importAxiom2.name;

          path.replaceWith({ type: type, name: name });
        } else if (matchesAxiom(path, path.node.name)) {
          path.replaceWith(types.nullLiteral());
        }
      }
    }
  };
};